#ifndef nameParser_H
#define nameParser_H

#define NOTSPEC 0   // status
#define UPTODATE 1   // status
#define UNSOLICITED 2     // status
#define OBSOLETE 3     // status
#define INVALID 4     // status
#define UNDEFINED 5     // status
#define ATOMIC 0   // type
#define GENERIC 1   // type
#define STATE 2    // type
#define RATE 3    // type
#define HISTOGRAM 4    // type
#define MODULE 5    // type
#define PORT 6
#define DEVICE 7
#define QUEUE 8
#define COMMANDDESC 9
#define INFO 10
#define HIDDEN 0   // visibility
#define VISIBLE 1   // visibility
#define MONITOR 2   // visibility
#define CHANGABLE 4    // visibility
#define IMPORTANT 8    // visibility
#define LOGGING 16    // visibility
#define NOMODE 0 // mode

class nameParser
{
  public:
  /**
   * Constructor and initialization
   */
  nameParser(){initParser(); parseQuality(0);}
  /**
   * Constructor passing a full namespec. Resets quality fields and parses fullname.
   * All fields will be set.
   * FullName: NameSpace/NodeSpec/ApplSpec/Name
   * NodeSpec: NodeName:NodeID
   * ApplSpec: [ApplNameSpace::]ApplNameID
   * ApplNameID: ApplName:ApplID
   */
  nameParser(char *fullname);
  /**
   * Following getters return the fields
   */
  char * getNameSpace(){return NameSpace;}
  char * getNodeSpec(){return NodeSpec;} // NodeName:NodeID
  char * getNodeName(){return NodeName;}
  char * getNodeID(){return NodeID;}
  char * getApplSpec(){return ApplSpec;} // [ApplNameSpace::]ApplNameID
  char * getApplName(){return ApplName;}
  char * getApplNameSpace(){return ApplNameSpace;}
  char * getApplNameID(){return ApplNameID;} // ApplName:ApplID
  char * getApplID(){return ApplID;}
  char * getName(){return Name;}
  char * getFullName(){return FullName;}
  /**
   * Parses fullname. All fields will be set. Returns false in case of error.
   * This is normally used to enter a new DIM DABC service name.
   */
  bool parseFullName(char *fullname=0);
  /**
   * Composes FullName from fields and returns it.
   * If a field has not been specified, NULL is returned.
   * Use this to get a new full name after setting some fields.
   */
  char * buildFullName();
  /**
   * Specify NameSpace field
   */
  void setNameSpace(char *nameserver=0){strcpy(NameSpace,nameserver);}
  /**
   * Specify NodeSpec field. Must be NodeName:NodeID
   * Parse and fill NodeName and NodeID.
   */
  void setNodeSpec(char *nodespec=0);
  /**
   * Specify NodeName. Build NodeSpec.
   */
  void setNodeName(char *nodename=0);
  /**
   * Specify NodeID. Build NodeSpec.
   */
  void setNodeID(char *nodeid=0);
  /**
   * Specify ApplSpec field. Must be [ApplNameSpace::]ApplName:ApplID.
   * Parse and fill ApplName and ApplID and optionally ApplNameSpace.
   */
  void setApplSpec(char *applspec=0);
  /**
   * Specify ApplName. Build ApplSpec.
   */
  void setApplName(char *applname=0);
  /**
   * Specify ApplNameID. Must be ApplName:ApplID.
   * Parse an fill ApplName and ApplID, then build ApplSpec.
   */
  void setApplNameID(char *applnameid=0);
  /**
   * Specify ApplID. Build ApplSpec.
   */
  void setApplID(char *applid=0);
  /**
   * Specify ApplNameSpace. Build ApplSpec.
   */
  void setApplNameSpace(char *applnamespace=0);
  /**
   * Specify Name
   */
  void setName(char *name=0){strcpy(Name,name);}
  /**
   * Returns true if all fields are set, false otherwise. Used by buildFullName.
   */
  bool checkFields();
  /**
   * Following getters return the information from quality fields.
   * parseQuality or buildQuality must have been called before.
   */
  unsigned int getType(){return type;}
  unsigned int getStatus(){return status;}
  unsigned int getVisibility(){return visibility;}
  unsigned int getMode(){return mode;}
  bool isUptoDate()    {return (status == UPTODATE);}
  bool isNotSpecified(){return (status == NOTSPEC);}
  bool isUndefined()   {return (status == UNDEFINED);}
  bool isAtomic()      {return (type   == ATOMIC);}
  bool isGeneric()     {return (type   == GENERIC);}
  bool isState()       {return (type   == STATE);}
  bool isInfo()        {return (type   == INFO);}
  bool isRate()        {return (type   == RATE);}
  bool isHistogram()   {return (type   == HISTOGRAM);}
  bool isCommandDescriptor()  {return (type   == COMMANDDESC);}
  bool isHidden()      {return (visibility == HIDDEN);}
  bool isVisible()     {return (visibility > 0);}
  bool isMonitor()     {return ((visibility & MONITOR) > 0);}
  bool isChangable()   {return ((visibility & CHANGABLE) > 0);}
  bool isImportant()   {return ((visibility & IMPORTANT) > 0);}
  bool isLogging()     {return ((visibility & LOGGING) > 0);}
  /**
   * Returns composed quality longword.
   */
  int getQuality(){return (int)quality;}
  /**
   * Composes quality longword from fields and returns it.
   */
  int buildQuality(unsigned int status, unsigned int type, unsigned int visib, unsigned int mode);
  /**
   * Sets fields from quality longword
   */
  void parseQuality(unsigned int qual);
  char * toString(){return FullName;}
  void printFields();
  void newCommand(char* name, char* scope);
  void addArgument(char* name, char* type, char* value, char* required);
  char * getXmlString();

private:
  void initParser();
  bool parseApplSpec();
  bool parseApplNameID();
  bool parseNodeSpec();
  void buildApplSpec();
  void buildNodeSpec();

private:
  char NameSpace[64];
  char NodeSpec[64];
  char NodeName[64];
  char NodeID[64];
  char ApplSpec[128];
  char ApplName[64];
  char ApplNameSpace[64];
  char ApplNameID[64];
  char ApplID[64];
  char Name[64];
  char FullName[256];
  char XmlString[512];
  char ArgString[128];
  bool finalized;
  unsigned int quality;
  unsigned int status;
  unsigned int visibility;
  unsigned int type;
  unsigned int mode;
};
#endif
